import toml
import fileinput
import sys

class ManipulateToml():
    def __init__(self, path, action, dependency, version):
        self.path = path
        self.action = action
        self.dependency = dependency
        self.version = version

        #load file
        self.data = toml.load(self.path)
        self.tools = []
        self.get_tools()
        self.project_name = self.get_project_name()
        self.primary_tool = self.get_primary_tool()

    def get_project_name(self):
        #for poetry
        if "poetry" in self.tools:
            return self.data['tool']['poetry']['name']

    def get_primary_tool(self):
        if "poetry" in self.tools:
            return "poetry"

    def get_tools(self):
        for key, val in self.data.items():
            for k, v in val.items():
                self.tools.append(k)

    def get_dependencies(self):
        return (self.data['tool'][self.primary_tool]['dependencies'])

    def get_names_of_dependencies(self):
        return list(self.get_dependencies().keys())

    def get_versions_of_dependencies(self):
        return list(self.get_dependencies().values())

    def get_number_of_dependencies(self):
        return len(self.get_dependencies())

    def check_for_specific_version(self, version):
        return self.get_versions_of_dependencies().count(version)

    def change_dep_version(self, dependency, version):
        if version == "X":
            version = "*"
        if dependency == "ALL":
            all_deps = self.get_names_of_dependencies()
            for i in range(len(all_deps)):
                self.data['tool'][self.primary_tool]['dependencies'][all_deps[i]] = version
        else:
            self.data['tool'][self.primary_tool]['dependencies'][dependency] = version

    def remove_dep(self, dependency):
        data = self.get_dependencies()
        try:
            data.pop(dependency)
        except KeyError:
            print("Dependency is not present in current dependency list!")

        return data

    def add_dep(self, dependency, version):
        data = self.get_dependencies()
        if version == "X":
            version = "*"

        data[dependency] = version
        return data

    def make_action(self):
        if self.action == "change":
            self.change_dep_version(self.dependency, self.version)
        elif self.action == "add":
            self.add_dep(self.dependency, self.version)
        elif self.action == "remove":
            self.remove_dep(self.dependency)

    def dump_to_file(self):
        with open(self.path, "w") as toml_file:
            toml.dump(self.data, toml_file)



from unittest import TestCase
from toml_adapt.ManipulateToml import ManipulateToml
import os

class SimpleBasicTestCase(TestCase):
    def setUp(self):
        self.__path = os.path.join(os.path.dirname(__file__), "data", 'pyproject.toml')
        self.__action = "add"
        self.__dependency = "niaclass"
        self.__version = "X"

    def test_make_simple_action(self):
         a = ManipulateToml(self.__path, self.__action, self.__dependency, self.__version)
         a.make_action()
         number_of_deps = a.get_number_of_dependencies()

         self.assertEqual(8, number_of_deps)




import toml
import fileinput
import sys

class ChangeLine():
    def __init__(self, path, action, old_line, new_line):
        self.path = path
        self.action = action
        self.old_line = old_line
        self.new_line = new_line

    def change_line(self):
        with fileinput.FileInput(self.path, inplace = True, backup ='.bak') as f:
            for line in f:
                if self.old_line + '\n' == line:
                    print(self.new_line, end ='\n')
                else:
                    print(line, end ='')



import argparse
import sys
from toml_adapt.ManipulateToml import ManipulateToml
from toml_adapt.ChangeLine import ChangeLine

def start():
    parser = argparse.ArgumentParser(description='Manipulate .toml files')

    parser.add_argument('-path',
        type=str,
        help="Path to the .toml file")

    parser.add_argument('-a',
        choices=['change', 'add', 'remove', 'change-line'],
        help="Select action")

    parser.add_argument('-dep',
        type=str,
        help="Dependency name")

    parser.add_argument('-ver',
        type=str,
        help="Version of dependency")

    parser.add_argument('-old',
        type=str,
        help="Old line")

    parser.add_argument('-new',
        type=str,
        help="New line")

    arguments = parser.parse_args()

    path = arguments.path
    action = arguments.a
    dependency = arguments.dep
    version = arguments.ver
    old_line = arguments.old
    new_line = arguments.new

    toml_manipulations = ['change', 'add', 'remove']
    other_manipulations = ['change-line']

    if action in toml_manipulations:
        a = ManipulateToml(path, action, dependency, version)
        a.make_action()
    elif action in other_manipulations:
        a = ChangeLine(path, action, old_line, new_line)
        a.change_line()

    if action in ['change', 'add', 'remove']:
        a.dump_to_file()
